/* $Id: rmat3supp.h 117 2008-08-06 14:53:40Z dirk $ */

/* GenLib2 - General Library 2
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef GENLIB2_RMAT3SUPP_INCLUDED
#define GENLIB2_RMAT3SUPP_INCLUDED

#include <ostream>
using namespace std;

namespace GenLib2
{

/** 3D rotation matrix support class.
 *  This class can be used to rotate a 3D coordinate system.
 *
 *  @author Dirk Stallmann
 */
class ERotMatrix3
{
public:
	/// type of rotation parameters
	enum ecode
	{
		/// undefined
		UNDEFINED,

		/// rotation matrix
		ROTATION_MATRIX,

		/// direction cosines
		DIRECTION_COSINES,

		/// rotation angles (omega-phi-kappa, fixed, clockwise)
		ANGLES_OPK_FIX,

		/// rotation angles (omega-phi-kappa, fixed, counter-clockwise)
		ANGLES_OPK_FIX_CCW,
		
		/// rotation angles (phi-omega-kappa, fixed, counter-clockwise)
		ANGLES_POK_FIX,

		/// rotation angles (omega-phi-kappa, rotated, counter-clockwise)
		ANGLES_OPK_ROT,

		/// rotation angles (phi-omega-kappa, rotated, counter-clockwise)
		ANGLES_POK_ROT,

		/// rotation angles (azimuth-elevation-roll, Australis)
		ANGLES_AUSTRALIS,

		/// "small" rotation angles
		ANGLES_SMALL,

		/// three upper-diagonal elements
		UPPER_DIAG_ELEM,

		/// Rodrigues matrix
		RODRIGUES_MATRIX,

		/// (Hamiltonian) unit quaternion
		UNIT_QUATERNION,

		/// one rotation about a line
		ROTATION_ABOUT_LINE,

		/// end-of-map
		END_OF_MAP
	};

	/// constructor
	ERotMatrix3() { }

	/// destructor
	virtual ~ERotMatrix3() { }

	/**@name get rotation parameter name.
	 *  @param code
	 *  @return name
	 */
	static const char* name(int code);

	/**@name get rotation parameter enumerator.
	 *  @param keyword
	 *  @return code
	 */
	static ecode code(const char* key);

	/**@name get number of rotation parameters.
	 *  @param code
	 *  @return number of parameters
	 */
	static size_t size(int code);

	/// print map
	void print(ostream& os) const;

	/// get number of map items
	size_t map_size() const;
	
private:
	/// map structure
	struct map_t
	{
		/// code
		int				code;

		/// keyword
		const char*		keyword;

		/// (long) name
		const char*		name;

		/// number of parameters
		size_t			size;
	};

	/// map
	static map_t _map[];
};


// non-member functions ...

} // namespace GenLib2

#endif /* GENLIB2_RM3SUPP_INCLUDED */
