/* $Id: uangle.cpp 117 2008-08-06 14:53:40Z dirk $ */

/* GenLib2 - General Library 2
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <cstring>
#include "uangle.h"

namespace GenLib2
{
const int UnitAngle::_map_size = 8;


// code, symbol (short name), long name, rho

UnitAngle::map_t UnitAngle::_map[] =
{
	{ UNKNOWN,	"unk",	"unkown",	0. },
	{ NONE,		"none",	"none",		1. },
	{ RADIAN,	"rad",	"radian",	1. },
	{ DEGREE,	"deg",	"degree",	180. / M_PI },
	{ HOUR,		"hr",	"hour",  	12. / M_PI },
	{ MINUTE,	"min",	"minute",	180. / M_PI / 60. },
	{ SECOND,	"sec",	"second",	180. / M_PI / 3600. },
	{ GON,		"gon",	"gon",		200. / M_PI }
};


// set

bool UnitAngle::set(const char* symbol)
{
	_code = code(symbol);
	if (_code == UNKNOWN) return false;
	
	_rho = rho(_code);

	return true;
}


// get code

UnitAngle::ecode UnitAngle::code(const char* symbol)
{
	for (int i = 0; i < _map_size; i++)
		{
		if (strcmp(symbol, _map[i].symbol) == 0)
			return (ecode)_map[i].code;
		}

	return (ecode)_map[0].code;
}


// get name

const char* UnitAngle::name(int code)
{
	for (int i = 0; i < _map_size; i++)
		{
		if (_map[i].code == code)
			return _map[i].name;
		}

	return _map[0].name;
}


// get symbol

const char* UnitAngle::symbol(int code)
{
	for (int i = 0; i < _map_size; i++)
		{
		if (_map[i].code == code)
			return _map[i].symbol;
		}

	return _map[0].symbol;
}


// get rho

double UnitAngle::rho(int code)
{
	for (int i = 0; i < _map_size; i++)
		{
		if (_map[i].code == code)
			return _map[i].rho;
		}

	return _map[0].rho;
}


// print map

void UnitAngle::print(ostream& os) const
{
	os << "code/keyword/name/rho\n";
	for (int i = 0; i < _map_size; i++)
		{
		os << _map[i].code << "/" << _map[i].symbol << "/"
			<< _map[i].name << "/" << _map[i].rho << endl;
		}
}

} // namespace GenLib2

