/* $Id: ulength.cpp 117 2008-08-06 14:53:40Z dirk $ */

/* GenLib2 - General Library 2
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include <cstring>
#include "ulength.h"

namespace GenLib2
{
const int UnitLength::_map_size = 12;


// code, symbol (short name), long name, scale

UnitLength::map_t UnitLength::_map[] =
{
	{ UNKNOWN,		"unk",	"unkown",		0. },
	{ KILOMETRE,	"km",	"kilometre",	1e+3 },
	{ METRE,		"m",	"metre",		1. },
	{ CENTIMETRE,	"cm",	"centimetre",	1e-2 },
	{ MILLIMETRE,	"mm",	"millimetre",	1e-3 },
	{ MICROMETRE,	"um",	"micrometre",  	1e-6 },
	{ MILE,			"mi",	"intern. mile",	1609.344 }, // exact
	{ MILES_US,		"miUS",	"US state mile",	1609.32721869 },
	{ INCH,			"in",	"inch",			0.0254 }, // exact
	{ FEET,			"ft",	"feet",			0.3048 }, // exact
	{ YARD,			"yd",	"yard",			0.9144 }, // exact
	{ NAUTICAL_MILE,	"n.m.",	"nautical mile",	1852. } // exact
};


// set unit using symbol-string

bool UnitLength::set(const char* symbol)
{
	_code = code(symbol);
	if (_code == UNKNOWN) return false;
	
	_scale = scale(_code);

	return true;
}


// get code

UnitLength::ecode UnitLength::code(const char* symbol)
{
	for (int i = 0; i < _map_size; i++)
		{
		if (strcmp(symbol, _map[i].symbol) == 0)
			return (ecode)_map[i].code;
		}

	return (ecode)_map[0].code;
}


// get name

const char* UnitLength::name(int code)
{
	for (int i = 0; i < _map_size; i++)
		{
		if (_map[i].code == code)
			return _map[i].name;
		}

	return _map[0].name;
}


// get symbol

const char* UnitLength::symbol(int code)
{
	for (int i = 0; i < _map_size; i++)
		{
		if (_map[i].code == code)
			return _map[i].symbol;
		}

	return _map[0].symbol;
}


// get rho

double UnitLength::scale(int code)
{
	for (int i = 0; i < _map_size; i++)
		{
		if (_map[i].code == code)
			return _map[i].scale;
		}

	return _map[0].scale;
}

} // namespace GenLib2

