/* $Id: vector3.h 117 2008-08-06 14:53:40Z dirk $ */

/* GenLib2 - General Library 2
 * Copyright (C) 2005 Dirk Stallmann
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#ifndef GENLIB2_VECTOR3_INCLUDED
#define GENLIB2_VECTOR3_INCLUDED

#include <iostream>
//#include "matrix.h"
//#include "math.h" // use own include file!

using namespace std;


// 2/3d Vector-Library

namespace GenLib2
{

template<class T>
class Vector3
{
public:
	// Constructors, destructor, assignment operator
	Vector3() { x=0; y=0; z=0; }
	Vector3(T s) { init(s); }
	Vector3(T x, T y, T z) { init(x,y,z); }
	Vector3(const Vector3& that) { init(that.x,that.y,that.z); }

	Vector3& init(T X, T Y, T Z) { x=X; y=Y; z=Z; return *this; }
	Vector3& init(T s) { x=s; y=s; z=s; return *this; }

    Vector3& operator=(const Vector3& a) {
		return init((x=a.x),(y=a.y),(z=a.z));
	}

    // composite assignment operators...
	/*
	Vector3& operator*=(T s) {
		x *= s; y *= s; z *= s;
		return *this;
	}
	*/

    Vector3& operator+=(const Vector3& a) {
		x += a.x; y += a.y; z += a.z;
		return *this;
	}

    Vector3& operator-=(const Vector3& a) {
		x -= a.x; y -= a.y; z -= a.z;
		return *this;
	}

    Vector3& operator*=(const Vector3& a) {
		x *= a.x; y *= a.y; z *= a.z;
		return *this;
	}

    Vector3& operator/=(const Vector3& a) {
		x /= a.x; y /= a.y; z /= a.z;
		return *this;
	}

/*
	// matrix-vector products...
	Vector3& prod(const Matrix<T>& A, const Vector3& b) {
		assert(A.rows() == 3);
		assert(A.cols() == 3);

		x = A(0,0) * b.x + A(0,1) * b.y + A(0,2) * b.z;
		y = A(1,0) * b.x + A(1,1) * b.y + A(1,2) * b.z;
		z = A(2,0) * b.x + A(2,1) * b.y + A(2,2) * b.z;

		return *this;
	}

	Vector3& prod_trn(const Matrix<T>& A, const Vector3& b) {
		assert(A.rows() == 3);
		assert(A.cols() == 3);

		x = A(0,0) * b.x + A(1,0) * b.y + A(2,0) * b.z;
		y = A(0,1) * b.x + A(1,1) * b.y + A(2,1) * b.z;
		z = A(0,2) * b.x + A(1,2) * b.y + A(2,2) * b.z;

		return *this;
	}
*/

	// data members
	T x, y, z;
};


// non-member implementations...

// arithmetic operators...

template<class T> inline
Vector3<T> operator+(const Vector3<T>& a, const Vector3<T>& b)
{
	Vector3<T> tmp(a.x + b.x, a.y + b.y, a.z + b.z);
	return tmp;
}

template<class T> inline
Vector3<T> operator-(const Vector3<T>& a, const Vector3<T>& b)
{
	Vector3<T> tmp(a.x - b.x, a.y - b.y, a.z - b.z);
	return tmp;
}

template<class T> inline
Vector3<T> operator*(const Vector3<T>& a, const Vector3<T>& b)
{
	Vector3<T> tmp(a.x * b.x, a.y * b.y, a.z * b.z);
	return tmp;
}

template<class T> inline
Vector3<T> operator/(const Vector3<T>& a, const Vector3<T>& b)
{
	Vector3<T> tmp(a.x / b.x, a.y / b.y, a.z / b.z);
	return tmp;
}


// scalar operators...

template<class T> inline
Vector3<T> operator*(const Vector3<T>& a, T s)
{
	Vector3<T> tmp(a.x * s, a.y * s, a.z * s);
	return tmp;
}

template<class T> inline
Vector3<T> operator/(const Vector3<T>& a, T s)
{
	Vector3<T> tmp(a.x / s, a.y / s, a.z / s);
	return tmp;
}


// comparison operators...

template<class T> inline
bool operator==(const Vector3<T>& a, const Vector3<T>& b)
{
  return (a.x == b.x && a.y == b.y && a.z == b.z);
}

template<class T> inline
bool operator!=(const Vector3<T>& a, const Vector3<T>& b)
{
  return (a.x != b.x || a.y != b.y || a.z != b.z);
}


// input operator, input could be x y z or (x,y,z)

template<class T> inline
istream& operator>>(istream& s, Vector3<T>& a)
{
	T x = 0, y = 0, z = 0;
	char  c = 0;

	s >> c;
	if (c == '(')
		{
		s >> x >> c;
		if (c == ',') s >> y >> c;
		if (c == ',') s >> z >> c;
		if (c != ')') s.clear(ios::badbit);
		}
	else
		{
		s.putback(c);
		s >> x >> y >> z;
		}

	if (s) a = Vector3<T>(x, y, z);
	return s;
}


// output operator

template<class T> inline
ostream& operator<<(ostream& s, const Vector3<T>& a)
{
	return s << "( " << a.x << ", " << a.y << ", " << a.z << ")";
}


// mathematical functions...

// absolute value

template<class T> inline
T abs2(const Vector3<T>& a)
{
	return (a.x * a.x + a.y * a.y + a.z * a.z);
}

template<class T> inline
T abs(const Vector3<T>& a)
{
	return sqrt(abs2(a));
}


// dot product

template<class T> inline
T dot(const Vector3<T>& a, const Vector3<T>& b)
{
	return (a.x * b.x + a.y * b.y + a.z * b.z);
}


// cross product

template<class T> inline
Vector3<T> cross(const Vector3<T>& a, const Vector3<T>& b)
{
	Vector3<T> tmp(
		a.y * b.z - a.z * b.y,
		-(a.x * b.z - a.z * b.x),
		a.x * b.y - a.y * b.x);
	return tmp;
}


// 2x2-matrix * 2-vector multiplication operator
/*
template<class T>
Vector2<T> operator*(const Matrix<T>& A, const Vector2<T>& b)
{
	assert(A.rows() == 2);
	assert(A.cols() == 2);
	
	Vector2<T> c;
	c.x = A(0,0) * b.x + A(0,1) * b.y;
	c.y = A(1,0) * b.x + A(1,1) * b.y;

    return c;
}
*/


// matrix-vector multiplication operator
/*
template<class T>
Vector3<T> operator*(const Matrix<T>& A, const Vector3<T>& b)
{
	assert(A.rows() == 3);
	assert(A.cols() == 3);
	
	Vector3<T> c;

	c.prod(A, b);
	//c.x = A(0,0) * b.x + A(0,1) * b.y + A(0,2) * b.z;
	//c.y = A(1,0) * b.x + A(1,1) * b.y + A(1,2) * b.z;
	//c.z = A(2,0) * b.x + A(2,1) * b.y + A(2,2) * b.z;

    return c;
}
*/

} // namespace GenLib2

#endif /* GENLIB2_VECTOR3_INCLUDED */

